
import * as jsDiff from 'diff';
import { unifyPath, hashCode } from './utils';
import * as rematch from './rematch';
import { ColorSchemeType, DiffStyleType, LineMatchingType, LineType, } from './types';
export const CSSLineClass = {
    INSERTS: 'd2h-ins',
    DELETES: 'd2h-del',
    CONTEXT: 'd2h-cntx',
    INFO: 'd2h-info',
    INSERT_CHANGES: 'd2h-ins d2h-change',
    DELETE_CHANGES: 'd2h-del d2h-change',
    IGNROE_CLASS: 'd2h-cntx',
};
export const defaultRenderConfig = {
    matching: LineMatchingType.NONE,
    matchWordsThreshold: 0.25,
    maxLineLengthHighlight: 10000,
    diffStyle: DiffStyleType.WORD,
    colorScheme: ColorSchemeType.LIGHT,
};
const separator = '/';
const distance = rematch.newDistanceFn((change) => change.value);
const matcher = rematch.newMatcherFn(distance);
function isDevNullName(name) {
    return name.indexOf('dev/null') !== -1;
}
function removeInsElements(line) {
    return line.replace(/(<ins[^>]*>((.|\n)*?)<\/ins>)/g, '');
}
function removeDelElements(line) {
    return line.replace(/(<del[^>]*>((.|\n)*?)<\/del>)/g, '');
}
function idFilter(origin_sql) {
    // id 过滤
    var res = /^(\d{18}|\d{16}|\d{14}|\d{12})$/
    return res.test(origin_sql)
}

function dateTimeFilter2(origin_sql) {
    // id 过滤, '2023-08-10 10:33:48.68311'
    var res = /^(\d{4}-\d{1,2}-\d{1,2}\s\d{1,2}:\d{1,2}:\d{1,2}.\d{1,5})$/
    return res.test(origin_sql)
}
function dateTimeFilter3(origin_sql) {
    // id 过滤,'2023-08-10 10:33:48'
    var res = /^(\d{4}-\d{1,2}-\d{1,2}\s\d{1,2}:\d{1,2}:\d{1,2})$/
    return res.test(origin_sql)
}

function nameFilter3(origin_sql) {
    // id 过滤,'2023-08-10 10:33:48'
    var res = /^(apitest\d+)$/
    return res.test(origin_sql)
}

function filterAll(origin_sql) {
    // 总过滤函数
    console.log("~~~~~~~diffhtml内部～～～")
    const idFlag = idFilter(origin_sql)
    const dtFlag = dateTimeFilter2(origin_sql)
    const dtFlag_2 = dateTimeFilter3(origin_sql)
    const nameFlag = nameFilter3(origin_sql)
    return idFlag || dtFlag || dtFlag_2 || nameFlag
}

function reBuildComponent(components) {
    for (var index =0; index < components.length; index ++) {
        console.log("fot~~~~~~~~~~")
        console.log(index)
        var nowComponent = components[index]
        var beforeComponent = undefined
        if (index > 0 && index < components.length - 1) {
            beforeComponent = components[index - 1]
        }
        console.log(beforeComponent)
        console.log(nowComponent)
        console.log("api~~~~~~~~~~~")
        if (nowComponent.added && beforeComponent.removed) {
            console.log("开始判断")
            console.log(nowComponent)
            console.log(beforeComponent)
            if (filterAll(nowComponent.value) && filterAll(beforeComponent.value)) {
                console.log("判断逻辑")
                delete nowComponent.added
                delete nowComponent.removed
                delete beforeComponent.added
                delete beforeComponent.removed
                beforeComponent.ignoreRight = true
                nowComponent.ignoreLeft = true
            }
        }
    }
}

export function toCSSClass(lineType) {
    switch (lineType) {
        case LineType.CONTEXT:
            return CSSLineClass.CONTEXT;
        case LineType.INSERT:
            return CSSLineClass.INSERTS;
        case LineType.DELETE:
            return CSSLineClass.DELETES;
    }
}
export function colorSchemeToCss(colorScheme) {
    switch (colorScheme) {
        case ColorSchemeType.DARK:
            return 'd2h-dark-color-scheme';
        case ColorSchemeType.AUTO:
            return 'd2h-auto-color-scheme';
        case ColorSchemeType.LIGHT:
        default:
            return 'd2h-light-color-scheme';
    }
}
function prefixLength(isCombined) {
    return isCombined ? 2 : 1;
}
export function escapeForHtml(str) {
    return str
        .slice(0)
        .replace(/&/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')
        .replace(/"/g, '&quot;')
        .replace(/'/g, '&#x27;')
        .replace(/\//g, '&#x2F;');
}
export function deconstructLine(line, isCombined, escape = true) {
    const indexToSplit = prefixLength(isCombined);
    return {
        prefix: line.substring(0, indexToSplit),
        content: escape ? escapeForHtml(line.substring(indexToSplit)) : line.substring(indexToSplit),
    };
}
export function filenameDiff(file) {
    const oldFilename = unifyPath(file.oldName);
    const newFilename = unifyPath(file.newName);
    if (oldFilename !== newFilename && !isDevNullName(oldFilename) && !isDevNullName(newFilename)) {
        const prefixPaths = [];
        const suffixPaths = [];
        const oldFilenameParts = oldFilename.split(separator);
        const newFilenameParts = newFilename.split(separator);
        const oldFilenamePartsSize = oldFilenameParts.length;
        const newFilenamePartsSize = newFilenameParts.length;
        let i = 0;
        let j = oldFilenamePartsSize - 1;
        let k = newFilenamePartsSize - 1;
        while (i < j && i < k) {
            if (oldFilenameParts[i] === newFilenameParts[i]) {
                prefixPaths.push(newFilenameParts[i]);
                i += 1;
            }
            else {
                break;
            }
        }
        while (j > i && k > i) {
            if (oldFilenameParts[j] === newFilenameParts[k]) {
                suffixPaths.unshift(newFilenameParts[k]);
                j -= 1;
                k -= 1;
            }
            else {
                break;
            }
        }
        const finalPrefix = prefixPaths.join(separator);
        const finalSuffix = suffixPaths.join(separator);
        const oldRemainingPath = oldFilenameParts.slice(i, j + 1).join(separator);
        const newRemainingPath = newFilenameParts.slice(i, k + 1).join(separator);
        if (finalPrefix.length && finalSuffix.length) {
            return (finalPrefix + separator + '{' + oldRemainingPath + ' → ' + newRemainingPath + '}' + separator + finalSuffix);
        }
        else if (finalPrefix.length) {
            return finalPrefix + separator + '{' + oldRemainingPath + ' → ' + newRemainingPath + '}';
        }
        else if (finalSuffix.length) {
            return '{' + oldRemainingPath + ' → ' + newRemainingPath + '}' + separator + finalSuffix;
        }
        return oldFilename + ' → ' + newFilename;
    }
    else if (!isDevNullName(newFilename)) {
        return newFilename;
    }
    else {
        return oldFilename;
    }
}
export function getHtmlId(file) {
    return `d2h-${hashCode(filenameDiff(file)).toString().slice(-6)}`;
}
export function getFileIcon(file) {
    let templateName = 'file-changed';
    if (file.isRename) {
        templateName = 'file-renamed';
    }
    else if (file.isCopy) {
        templateName = 'file-renamed';
    }
    else if (file.isNew) {
        templateName = 'file-added';
    }
    else if (file.isDeleted) {
        templateName = 'file-deleted';
    }
    else if (file.newName !== file.oldName) {
        templateName = 'file-renamed';
    }
    return templateName;
}
export function diffHighlight(diffLine1, diffLine2, isCombined, config = {}) {
    console.log('处理diff行行行行行行行行行行行行行行行行行行行行行行行行行行行行行行');
    const { matching, maxLineLengthHighlight, matchWordsThreshold, diffStyle } = Object.assign(Object.assign({}, defaultRenderConfig), config);
    const line1 = deconstructLine(diffLine1, isCombined, false);
    const line2 = deconstructLine(diffLine2, isCombined, false);
    console.log(line1);
    console.log(line2);
    console.log(diffStyle);
    console.log('line~~~~~~~~~~~~~~~~~~~~~');
    if (line1.content.length > maxLineLengthHighlight || line2.content.length > maxLineLengthHighlight) {
        console.log('hhhhhhh');
        return {
            oldLine: {
                prefix: line1.prefix,
                content: escapeForHtml(line1.content),
            },
            newLine: {
                prefix: line2.prefix,
                content: escapeForHtml(line2.content),
            },
        };
    }
    const diff = diffStyle === 'char'
        ? jsDiff.diffChars(line1.content, line2.content)
        : jsDiff.diffWordsWithSpace(line1.content, line2.content);
    console.log('二层diffffff');
    console.log(diff);
    console.log(1111);
    const changedWords = [];
    if (diffStyle === 'word' && matching === 'words') {
        console.log('jhdjdjhdhjdhjdhjdjhdjd~~~~~~~~~~');
        const removed = diff.filter(element => element.removed);
        const added = diff.filter(element => element.added);
        const chunks = matcher(added, removed);
        chunks.forEach(chunk => {
            if (chunk[0].length === 1 && chunk[1].length === 1) {
                const dist = distance(chunk[0][0], chunk[1][0]);
                if (dist < matchWordsThreshold) {
                    changedWords.push(chunk[0][0]);
                    changedWords.push(chunk[1][0]);
                }
            }
        });
    }
    console.log(22222);
    console.log(diff);
    console.log('~~~~~start to reduce~~~~');
    reBuildComponent(diff)
    console.log("22222~~~~~~~~~~~222222222")
    console.log(diff)

    const highlightedLine = diff.reduce((highlightedLine, part) => {
        console.log('12781287781278127812');
        console.log(highlightedLine);
        console.log(part);
        console.log('@@@@@@@@@@*******@@@@@@@@@@');
        const elemType = part.added
            ? 'ins'
            : part.removed
                ? 'del'
                : part.ignoreRight
                    ? 'igr'
                    : part.ignoreLeft
                        ? 'igl'
                        : null;
        console.log(elemType);
        const addClass = changedWords.indexOf(part) > -1 ? ' class="d2h-change"' : '';
        console.log(addClass);
        const escapedValue = escapeForHtml(part.value);
        console.log(escapedValue);
        return elemType !== null
            ? `${highlightedLine}<${elemType}${addClass}>${escapedValue}</${elemType}>`
            : `${highlightedLine}${escapedValue}`;
    }, '');
    console.log(33333);
    console.log(highlightedLine);
    console.log('?????');
    console.log('diff line 拼接');
    const oldContent = removeInsElements(highlightedLine);
    const newContent = removeDelElements(highlightedLine);
    if (oldContent.indexOf('<del>') === -1 &&
        oldContent.indexOf('<ins>') === -1 &&
        newContent.indexOf('<del>') === -1 &&
        newContent.indexOf('<ins>') === -1) {
        return {
            oldLine: {
                prefix: undefined,
                content: escapeForHtml(line1.content),
            },
            newLine: {
                prefix: undefined,
                content: escapeForHtml(line2.content),
            },
        };
    }
    else {
        return {
            oldLine: {
                prefix: line1.prefix,
                content: oldContent,
            },
            newLine: {
                prefix: line2.prefix,
                content: newContent,
            },
        };
    }
}
//# sourceMappingURL=render-utils.js.map